namespace EnotaryoPH.Web.Common.Services
{
    public interface IVideoConferenceService
    {
        Task ApproveTransactionAsync(Guid transaction_UID);

        bool CanStart(Guid transaction_UID);

        Guid GetUIDByTransactionUID(Guid transaction_UID);

        bool HasExpired(Guid transaction_UID);

        Task<Guid> StartAsync(Guid transaction_UID);

        Task StartRecordingAsync(Guid transaction_UID, string serverCallID);
    }
}